@@ -1,8 +1,8 @@
 package Cassandra::Client::Connection;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Connection::VERSION = '0.13_009'; # TRIAL
+$Cassandra::Client::Connection::VERSION = '0.14_001'; # TRIAL
 
-$Cassandra::Client::Connection::VERSION = '0.13009';use 5.010;
+$Cassandra::Client::Connection::VERSION = '0.14001';use 5.010;
 use strict;
 use warnings;
 use vars qw/$BUFFER/;
@@ -70,9 +70,13 @@ sub new {
         pending_write   => undef,
         shutdown        => 0,
         read_buffer     => \(my $empty= ''),
+        bytes_sent      => 0,
+        bytes_read      => 0,
 
         tls             => undef,
         tls_want_write  => undef,
+
+        healthcheck     => undef,
     }, $class;
     weaken($self->{async_io});
     weaken($self->{client});
@@ -761,10 +765,12 @@ sub request {
             my $length= length $data;
             my $rv= Net::SSLeay::write(${$self->{tls}}, $data);
             if ($rv == $length) {
+                $self->{bytes_sent} += $rv;
                 # All good
             } elsif ($rv > 0) {
                 # Partital write
                 substr($data, 0, $rv, '');
+                $self->{bytes_sent} += $rv;
                 $self->{pending_write}= $data;
                 $self->{async_io}->register_write($self->{fileno});
             } else {
@@ -802,9 +808,11 @@ sub request {
             my $length= length $data;
             my $result= syswrite($self->{socket}, $data, $length);
             if ($result && $result == $length) {
+                $self->{bytes_sent} += $result;
                 # All good
             } elsif (defined $result || $! == EAGAIN) {
                 substr($data, 0, $result, '') if $result;
+                $self->{bytes_sent} += $result;
                 $self->{pending_write}= $data;
                 $self->{async_io}->register_write($self->{fileno});
             } else {
@@ -848,6 +856,7 @@ READ:
                 $BUFFER .= $bytes;
                 $bufsize += $rv;
                 $should_read_more= 1;
+                $self->{bytes_read} += $rv;
             }
 
             if ($rv <= 0) {
@@ -875,6 +884,7 @@ READ:
             if ($read_cnt) {
                 $bufsize += $read_cnt;
                 $should_read_more= 1 if $read_cnt >= 16384;
+                $self->{bytes_read} += $read_cnt;
 
             } elsif (!defined $read_cnt) {
                 if ($! != EAGAIN) {
@@ -944,6 +954,7 @@ sub can_write {
         my $rv= Net::SSLeay::write(${$self->{tls}}, $self->{pending_write});
         if ($rv > 0) {
             substr($self->{pending_write}, 0, $rv, '');
+            $self->{bytes_sent} += $rv;
             if (!length $self->{pending_write}) {
                 $self->{async_io}->unregister_write($self->{fileno});
                 delete $self->{pending_write};
@@ -981,6 +992,7 @@ sub can_write {
         }
         if ($result == 0) { return; } # No idea whether that happens, but guard anyway.
         substr($self->{pending_write}, 0, $result, '');
+        $self->{bytes_sent} += $result;
 
         if (!length $self->{pending_write}) {
             $self->{async_io}->unregister_write($self->{fileno});
@@ -1000,9 +1012,39 @@ sub can_timeout {
         is_timeout      => 1,
         request_error   => 1,
     ));
+    $self->maybe_healthcheck;
     return;
 }
 
+sub maybe_healthcheck {
+    my ($self)= @_;
+    return if $self->{healthcheck};
+    my $check= $self->{healthcheck}= {};
+
+    # Mark how many bytes we've read right now
+    $check->{bytes_read}= $self->{bytes_read};
+
+    series([
+        sub {
+            my ($next)= @_;
+            $self->request($next, OPCODE_OPTIONS, '');
+        }
+    ], sub {
+        my ($error)= @_;
+        $self->{healthcheck}= undef;
+        if ($self->{bytes_read} > $check->{bytes_read}) {
+            # Don't care what happened, the connection is still fine
+        } elsif (!$error) {
+            # All good
+        } elsif (is_blessed_ref($error) && $error->is_request_error && $error->is_timeout) {
+            # Not good.
+            $self->shutdown("health check timed out");
+        } else {
+            # No clue. :-)
+        }
+    });
+}
+
 sub shutdown {
     my ($self, $shutdown_reason)= @_;
 
@@ -1103,7 +1145,7 @@ Cassandra::Client::Connection
 
 =head1 VERSION
 
-version 0.13_009
+version 0.14_001
 
 =head1 AUTHOR
 
