@@ -1,13 +1,14 @@
 package Cassandra::Client::Error::Base;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Error::Base::VERSION = '0.13_009'; # TRIAL
+$Cassandra::Client::Error::Base::VERSION = '0.14_001'; # TRIAL
 
-$Cassandra::Client::Error::Base::VERSION = '0.13009';use 5.010;
+$Cassandra::Client::Error::Base::VERSION = '0.14001';use 5.010;
 use strict;
 use warnings;
 
 sub new { my $class= shift; bless { code => -1, message => "An unknown error occurred", @_ }, $class }
-use overload '""' => sub { "Error $_[0]{code}: $_[0]{message}" };
+use overload '""' => sub { $_[0]->to_string };
+sub to_string { "Error $_[0]{code}: $_[0]{message}" }
 sub code { $_[0]{code} }
 sub message { $_[0]{message} }
 sub is_request_error { $_[0]{request_error} }
@@ -26,7 +27,7 @@ Cassandra::Client::Error::Base
 
 =head1 VERSION
 
-version 0.13_009
+version 0.14_001
 
 =head1 AUTHOR
 
